#ifndef HASH_FUNC_H_INCLUDED
#define HASH_FUNC_H_INCLUDED


#include "types.h"

// Basic blocks of hash function 

//void X_64(u32 *hash, u32 const * data);  // 64 bit data; 64 bit hash
//void I_64(u32 *hash, u32 const * data);  // 64 bit data; 64 bit hash
//void X_128(u32 *hash, u32 const * data);  // 64 bit data; 128 bit hash
//void I_128(u32 *hash, u32 const * data);  // 64 bit data; 128 bit hash
void XI_256(u32 *hash, u32 const *data); //  64 bit data; 256 bit hash

void hash_block_256(u32 *hash, u32 const *data); // 64 bit data; 256 bit hash 

void Hash_Block_X_64(u32 *hash, u8 const* data, u32 data_length_bytes); // 8 bit, any length


void Hash_Block_256(u32 *hash, u8 const* data, u32 data_length_bytes); // Any length
void Hash_Block_256(u32 *hash, u32 const* data, u32 data_length_x32); // Any length


// Invertible scrambling and unscrambling

void mix_data(u32 *x, const u32 len_x32, const u32 rounds = 4);
void unmix_data(u32 *x, const u32 len_x32, const u32 rounds = 4);

void xor_data(u32 *x, u32 const *xor_data, const u32 len);

//-------------------------
// Universal hash function
// hash size multiple of 128 bits (4 dwords)
// Any size starting from 128 bits
// Concept similar to SKEIN, however internals are different
// Speed doesn't matter as DH computation takes ~1000 times more cpu.
// Also, it would be difficult to implement this function in hardware
// 
typedef struct 
{
u32 seed[2];    
} COLLAPSE_TWEAK_T;

class COLLAPSE
 {
 static const u32 collapse_xxtea_rounds = 66;
 static const u32 collapse_mix_rounds = 16;
  
 u32 size_x32;
 u32 iter_per_block;

 COLLAPSE_TWEAK_T tweak;  

 u32 *data_input;
 u32 *lock_bar;
 
 void prng(u32 *output);

 void process_hash( u32 *hash, 
                    void const *data, u32 current_data_block_size_bytes, 
                    u32 total_length_bytes,
                    COLLAPSE_TWEAK_T *tweak_ptr,
                    int initialize_hash);
   
    
public:
    COLLAPSE(u32 hash_size_x32); // a multiple of 4 dwords
    ~COLLAPSE();

    void hash(u32 *hash, void const *data, u32 current_data_block_size_bytes); // Just one block
    void start_hash(u32 *hash, void const *data, u32 current_data_block_size_bytes, u32 total_length_bytes, COLLAPSE_TWEAK_T *tweak); // 1-st block of multivolume
    void continue_hash(u32 *hash, void const *data, u32 current_data_block_size_bytes, COLLAPSE_TWEAK_T *tweak); // next blocks of multivolume

    // Can be used as block cipher for testing

    void encrypt(u32 *data, u32 *key, COLLAPSE_TWEAK_T *tweak);   
    void decrypt(u32 *data, u32 *key, COLLAPSE_TWEAK_T *tweak);   

    void reset();
 };


class DH_HASH
 {
 COLLAPSE *collapse;
       
public:
    DH_HASH();
    ~DH_HASH();
    void hash(u32 *hash, u32 const* g, u32 const* y, u32 const* p);
    void reset();
        
 };


//-------------------------------------------
// This gives full 32-bit diffusion in 4 iterations
// Do NOT change -> a lot of things use it
//
static inline u32 k_mix(u32 x)
{
x ^= 0xDF15E92;
x = (x << 5) - x;
x ^= x >> 9;
x = (x << 19) | (x >> 13);
return x;
}
//---------------------
// Mini hash function
//
static inline u32 k_32(u32 x, u32 y)
{
return x ^ k_mix(x + y);
}

#endif

