// Montgomery algorithm for fast calculation g^x mod P

#include "montgomery.h"
#include "random.h"
//#include <stdio.h>

/*
u32 test_monpro(u32 a_, u32 b_, u32 n, u32 n_, u32 r)
{
u32 t = a_* b_; // 16 x 16 = 32 bit
u64 uu = (((t*n_)%r)*n) + (u64)t; // 33 bit!

u32 u = (u32)(uu/r);
if(u >= n) u -= n;

return u;
}


// ab mod n
// n must be odd
u32 test_montgomery(u32 a, u32 b, u32 n)
{
u32 r = 65536;

// Convert 16-bit numbers to Montgomery form

u32 a_ = (a*r)%n;
u32 b_ = (b*r)%n;


// n_ = - n^-1 modulo R
// dumb search

u32 n_;
for(n_ = 1; n_ <= 65535; n_++)
 {
 if(((n*n_) & 0xFFFF) == 1) break;
 }

n_ = 65536 - n_;

u32 u = test_monpro(a_, b_, n, n_, r);

//printf("\n a_ = %u  b_ = %u  u = %u", a_, b_, u);


// demontgomerize

u = test_monpro(u, 1, n, n_, r);

return u;
}
*/
//------------------------------------
// Length must be power of 2
//
MONTGOMERY_POW_MOD::MONTGOMERY_POW_MOD(const u32 length_x32)
{
len = length_x32;    

tmp0 = new u32[2*len];
tmp1 = new u32[2*len];
tmp2 = new u32[2*len];
tmp3 = new u32[2*len];

tmp_result = new u32[len];
pwr_x = new u32[len];
n_  = new u32[len];

t = new u32[2*len];
u = new u32[2*len];

p_is_precalculated = 0;
p_precalculated = new u32[len];


fast_mul  = new FAST_MUL(len);
fm = new FAST_MODULO(len);
inverse_modulo = new INVERSE_MODULO(len + 1); 
}
//-----------------------------

MONTGOMERY_POW_MOD::~MONTGOMERY_POW_MOD()
{
reset();    

delete inverse_modulo;
delete fm;
delete fast_mul;

delete[] p_precalculated;

delete[] u;
delete[] t;

delete[] n_;
delete[] pwr_x;
delete[] tmp_result;

delete[] tmp3;
delete[] tmp2;
delete[] tmp1;
delete[] tmp0;
}

void MONTGOMERY_POW_MOD::reset()
{
cheap_random_number(tmp0, 2*len);
cheap_random_number(tmp1, 2*len);
cheap_random_number(tmp2, 2*len);
cheap_random_number(tmp3, 2*len);

cheap_random_number(tmp_result, len);
cheap_random_number(pwr_x, len);
cheap_random_number(n_, len);

cheap_random_number(t,2*len);
cheap_random_number(u,2*len);

p_is_precalculated = 0;
cheap_random_number(p_precalculated, len);


fast_mul->reset();
fm->reset();
inverse_modulo->reset();    
}
//------------------------------
// Calculate Montgomery product of montgomerized numbers
//
void MONTGOMERY_POW_MOD::monpro(u32 *result, u32 const* a, u32 const* b)
{
// t = a_ b_;
// u = (t + ((t*n_)%r)*n)/r;
// if(u >= n) u -= n;
    

fast_mul->fast_mul(t, a, b); // t = a_ b_

fast_mul->fast_mul(u, t, n_); // u = (t * n_) mod r

fast_mul->fast_mul(tmp0, u, n); // tmp0 = u * n

u32 c = multi_add(u, tmp0, t, 2*len); // u = tmp0 + t

u32 cs = multi_sub(t, u + len, n, len); // (u/r) mod n

if((c) || (!cs)) 
 {
 copy(result, t, len);
 }
else 
 {
 copy(result, u + len, len);
 }
}

//---------------------
// Note: p must be odd
// Montgomery calculation (X^y)mod P
// It is about 1.9 times faster then direct calculation
//
void MONTGOMERY_POW_MOD::multi_pow_mod(u32 *result, const u32 *x, const u32 *y, const u32 *p)   // (x ^ y) modulo p
{
MONTGOMERY_POW_MOD::n = p;

int precalculated = 0;

if((p_is_precalculated) && (0 == multi_compare(p, p_precalculated,len))) 
 {
 precalculated = 1;
 }
else
 {
 p_is_precalculated = 1;
 copy(p_precalculated, p, len);   
 precalculated = 0;
 }

if(!precalculated) 
 {
 fm->fast_modulo_init(p);
 }

// 
// calculate r mod p (can't be done by fast modulo, because r is too long)
// 
zero(tmp0, 2*len);
tmp0[len] = 1; 

zero(tmp1, 2*len);
copy(tmp1, p, len);

multi_div(tmp3, tmp2 , tmp0, tmp1, 2*len);  // tmp2 = 1 * r mod p
copy(tmp_result, tmp2, len); 

// Calculate (r x) mod p

fast_mul->fast_mul(tmp0, tmp_result, x);
fm->fast_modulo(pwr_x, tmp0);

if(!precalculated)
 {

 // Calculate n_ = -(n^-1) mod r

 zero(tmp0, len);
 tmp0[len] = 1;   // tmp0 = r = 2^k

 copy(tmp1, p, len); // tmp1 = p
 tmp1[len] = 0;

 inverse_modulo->inverse(tmp2, tmp1, tmp0);
 multi_sub(n_, tmp0, tmp2, len); // /-/ inverse
 }

// raising to power

for(u32 ci = 0; ci < (len * 32); ci++)
 { 
 if(multi_get_bit(y,ci)) 
  {
  monpro(tmp_result, tmp_result, pwr_x);
  }

 // Calculate power(x) modulo p    

 monpro(pwr_x, pwr_x, pwr_x); // X^2 
 }

// Convert result back to normal representation

zero(tmp0, len);
tmp0[0] = 1;

monpro(result, tmp_result, tmp0);
}
//=============================

MONPRO::MONPRO(u32 length_x32)
{
len = length_x32;    
u = new u32[2*len];
t = new u32[2*len];
acc = new u32[2*len];
n  = new u32[len];
n_ = new u32[len];
fast_mul = new FAST_MUL(len); 
inverse_modulo = new INVERSE_MODULO(len + 1);
}

MONPRO::~MONPRO()
{
delete inverse_modulo;    
delete fast_mul;
delete[] n_;
delete[] n;    
delete[] acc;
delete[] t;
delete[] u;
}

//------------------------------
// Calculate Montgomery product of montgomerized numbers
//
void MONPRO::monpro(u32 *result, u32 *const a, u32 *const b)
{
// t = a_ b_;
// u = (t + ((t*n_)%r)*n)/r;
// if(u >= n) u -= n;
    

fast_mul->fast_mul(t, a, b); // t = a_ b_

fast_mul->fast_mul(u, t, n_); // u = (t * n_) mod r

fast_mul->fast_mul(acc, u, n); // acc = u * n

u32 c = multi_add(u, acc, t, 2*len); // u = tmp0 + t

u32 cs = multi_sub(t, u + len, n, len); // (u/r) mod n

if((c) || (!cs)) 
 {
 copy(result, t, len);
 }
else 
 {
 copy(result, u + len, len);
 }
}
//---------------------------------------------
// p must be odd!

void MONPRO::monpro_init(u32 const *p)
{
copy(n, p, len);

// Calculate n_ = -(n^-1) mod r

zero(t, len);
t[len] = 1;   // t = r = 2^k

copy(u, p, len); // u = p
u[len] = 0;

inverse_modulo->inverse(acc, u, t);

multi_sub(n_, t, acc, len); // /-/ inverse
}

// Convert x to number from montgomery form

void MONPRO::demontgomerize(u32 *x)
{
zero(u, len);
u[0] = 1;
    
MONPRO::monpro(x, x, u); 
}

// Convert x to montgomery form from number

void MONPRO::montgomerize(u32 *x)
{
zero(u, len);
copy(u+len, x, len);

copy(t, n, len);
zero(t+len, len);

multi_div(acc, x, u, t, 2*len); // x_ = (x * r) mod P
}


//============================ <END> =============================

