#ifndef RANDOM_H_INCLUDED
#define RANDOM_H_INCLUDED
#include "types.h"

// Generate strong pseudorandom numbers


void random_number(u32 *x, const u32 length_x32);


//----------------------------------
// Collect entropy into main seed
//
void collect_entropy(); // system provided
void collect_entropy(u32 const* data); // data u32[2] for legacy compatibility
void collect_entropy(void const * data, const u32 data_size_bytes);



void reset_stack(void);

u32 cheap_random_number();
void cheap_random_number(u32 *x, const u32 length_x32);
void cheap_random_number(u8 *x, const u32 size_of_x_bytes);


void initialize_seed(void); // Initialize main seed file



u32 lcg64(u32 *state);
u32 nlcg128(u32 *state);


void keystream_generator(u32 *output, u32 *state);
void keystream_generator_init(u32 *state, u32 const *key);


class KEYSTREAM_GENERATOR
{
u8 byte_buffer[8];
u32 byte_buffer_idx;
u32 state[12];
u64 cntr;

public:
    KEYSTREAM_GENERATOR();
    ~KEYSTREAM_GENERATOR();
    
    void init_key(u32 const* key); // Initalize with 384-bit key
    u8 get_keystream_byte(); // Generate pseudorandom byte
    int counter(u64 cnt); // returns 0 if counter less then current (can't go back in time)
    u64 counter(); // returns counter
    void reset();  

};




#endif

